unit ucustomcomm;
//Custom commands
{$mode ObjFPC}{$H+}


interface

uses
  Classes, SysUtils, CadApi, Windows, Dialogs, Math;

const
SHAPE_USR_CMD = 1;
//DIM_TAG_COORD_CMD = 2;

CMD_DOSHAPEPROP = CAD_CMD_CUSTOM + SHAPE_USR_CMD;
//CMD_DOTAGCOORD = CAD_CMD_CUSTOM + DIM_TAG_COORD_CMD;
STR_ERROR_FAIL_CUST_SHAPE_COM = 'Error fail to create shape properties command';


type

TRPoints = Record
  X, Y: double;
end;

TShapeData = record
  Area, Perimeter, Angle, Cx, Cy, Cz,
  dx, dy, MsX, MsY,    //width and height and  Static moments
  //local data
  IX, IY, IXY, J,
  rIX, rIY, rJ,
  //Global data
  IXgl, IYgl, IXYgl, Jgl,
  rIXgl, rIYgl, rJgl
  :Double;
end;





TLocalShapeData = record
 //Shapes principal Geometric properties datum
 ShapeFull: boolean;
 iEnt: integer;  //Shape type
 Width, Height, Radius, Angle,  // primary shape data
 Area, Perimeter, MaxX, MinX, MaxY, Miny,
 //Local data
 Xcg, Ycg, Zcg, IXcg, IYcg, IXYcg, IZZcg,
 rxcg, rycg, rocg,
 WxUpcg, WxDowncg, WyLeftcg, WyRightcg, // Section modulus from local cg
 WxMax, WyMax,                          // Max_x and Max_Y
 //principal moments os inertia
 I1,I2,Theta,
 IUcg, IVcg, IUVcg, IJUVcg,
 rucg, rvcg, rouvcg,
//Global coords data
 Xax, Yax, Zax,   //shapes center Coords local and global
 IXax, IYax, IXYax, IZZax,
 rxax, ryax, roax,
 MsX, MsY  //static moments
 : Double;
 end;
PLocalShapeData = ^TLocalShapedata; //Pointer to type TShapedata data

 //Create command
procedure CreateShapePropCommand;
//Do (execute) command
procedure DoShapeProp; stdcall;
// drag command (draw while dragging)
procedure DragShapeProp(hDwg: Integer; nPoints: Integer; x, y, z : Double); stdcall;
procedure ExecuteShapeProp(nEnts: Integer);
procedure SolvePolyline(hEnt: Thandle);
procedure Solverect(hEnt: Thandle);
procedure SolveCircle(hEnt: Thandle);
procedure SolveEllipse(hEnt: Thandle);


var
 points: array of TRPoints; // dynamic array of TRPoints
 bFullShape: Boolean;
 ComposedShapeData: TLocalShapeData; //composed shape result will be stored here
 custPrecision: Integer;
 custUnit: string[6];

implementation

Uses
  uGeoProps;



{
 procedure TagPoint(hEnt: Thandle);
 var
  x, y, z: Double;
  hAttrib: Thandle;
  s: String;
  b: TStrBuff;
 begin
  if hEnt = 0 then exit; //nothing to do
  hBlock:= CadGetBlockByName(hDwg, 'vertextag');
  IdBlock:= CadBlockGetID(hBlock);
  CadPointGetCoord(hEnt, @X, @Y, @Z);

  hblock:= CadAddInsBlock(hDwg, IdBlock, X, Y, Z, 1, 0);

  hAttrib:= CadInsBlockGetFirstAtt(hBlock);
  s:= '('+FloatToStrF(X,ffGeneral,4,4)+','+FloatToStrF(Y,ffGeneral,4,4)+')';
  CadAttPutValue (hAttrib, StrToBuffPtr(s,b));
  CadUpdate(hdwg);
  CadWndRedraw(hWND);
 end;

 procedure TagLine(hEnt: Thandle);
 begin

 end;

 procedure TagPoly(hEnt: Thandle);
 begin

 end;

 procedure TagRect(hEnt: Thandle);
 begin

 end;
}

procedure SolvePolyline(hEnt: Thandle);
var
  i, j, iVert: UINT;
  x, y, z,  ai, P, dx, dy: double;
  aPoint, C: TRPoints;
  rdata: TLocalShapeData;

begin
 CadCmdPrompt(hDwg, 'solving polyline', 'xxxx');
 iVert:= CadPlineGetNumVers(hEnt);
 if iVert < 1 then exit; //nothing to do
 if not Boolean(CadPlineGetClosed(hEnt)) then begin
   //if unclosed polyline then nothing to do
   CadCmdPrompt(hDwg, 'unclosed polyline. Nothing to do!', 'xxxx');
   exit;
 end;
 // begin calc polyline
 Fillchar(points,Sizeof(points),0); // erase all points array data
 for i:= 0 to iVert -1 do begin
     SetLength(points, iVert);
     CadPlineGetVer(hEnt, i, @x, @y, @z);
     aPoint.X:= x;
     aPoint.Y:= y;
     points[i]:= aPoint; //dynamic array index is zero based!! so (i-1)
 end;

 Fillchar(rdata,Sizeof(rdata),0); //erase rdata contents
 rdata.iEnt:= CAD_ENT_POLYLINE;
 rdata.ShapeFull:= bFullShape;  //gets if shape is full or void
 C.X:=0;
 C.Y:=0;
 P:=0;
 dx:=0;
 dy:=0;
 rdata.Angle:= CadRectGetAngle(hEnt); //General angle must be provided by onrotatedent event
 For i:= 0 to iVert-1 do begin
    j:=(i + 1) mod iVert;
    rdata.Area := rdata.Area + (Points[i].X * Points[j].Y - Points[j].X * Points[i].Y)/2;
    ai:= Points[i].X*Points[j].Y-Points[j].X*Points[i].Y;
    rdata.IXax:= rdata.IXax + (Sqr(Points[i].Y)+Points[i].Y*Points[j].Y+Sqr(Points[j].Y))*ai/12;
    rdata.IYax:= rdata.IYax + (Sqr(Points[i].x)+Points[i].X*Points[j].X+Sqr(Points[j].X))*ai/12;
    rdata.IXYax:= rdata.IXYax + (Points[i].X*Points[j].Y+2*Points[i].X*Points[i].Y+2*Points[j].X*Points[j].Y+Points[j].X*Points[i].Y)*ai/24;
    P:= Points[i].X * Points[j].Y - Points[j].X * Points[i].Y;
    C.X := C.X + (Points[i].X + Points[j].X) * P;
    C.Y := C.Y + (Points[i].Y + Points[j].Y) * P;
    dx:= Abs(Points[j].X-Points[i].X);
    dy:= Abs(Points[j].Y-Points[i].Y);
    rdata.Perimeter:= rdata.Perimeter+Sqrt(Sqr(dx)+Sqr(dy));
  end;

  rdata.MaxY:= CadEntityGetTop(hEnt);
  rdata.MinY:= CadEntityGetBottom(hEnt);
  rdata.MaxX:= CadEntityGetRight(hEnt);
  rdata.MinX:= CadEntityGetLeft(hEnt);

  C.X := C.X / (6 * rdata.Area);
  C.Y := C.Y / (6 * rdata.Area);
  rdata.Xax:=C.X;
  rdata.Yax:=C.Y;

  if (rdata.ShapeFull and (rdata.Area < 0)) or ((rdata.Area > 0) and (not rdata.ShapeFull) ) then begin   // se integração for negativa então inverte o sinal
     rdata.Area:= -rdata.Area;
     rdata.IXax:= -rdata.IXax;
     rdata.IYax:= -rdata.IYax;
     rdata.IXYax:= -rdata.IXYax;
  end;

  //get min and max coords
  rdata.MaxY:= CadEntityGetTop(hEnt);
  rdata.MinY:= CadEntityGetBottom(hEnt);
  rdata.MaxX:= CadEntityGetRight(hEnt);
  rdata.MinX:= CadEntityGetLeft(hEnt);

  rdata.IXcg:= rdata.IXax-Sqr(C.Y)*rdata.Area;
  rdata.IYcg:= rdata.IYax-Sqr(C.X)*rdata.Area;
  rdata.IXYcg:= rdata.IXYax - C.X*C.Y*rdata.Area;

  rdata.MsX:= rdata.Area * rdata.Yax; //static moment over x
  rdata.MsY:=rdata.Area * rdata.Xax; //static moment round y


  rdata.IZZcg:= rdata.IXcg+rdata.IYcg; //Polar momento of inertia
  rdata.IZZax:= rdata.IXax+rdata.IYax;

  rdata.rxcg:= Sqrt(rdata.IXcg/rdata.Area); //radii of giration shape
  rdata.rycg:= Sqrt(rdata.IYcg/rdata.Area);
  rdata.rocg:= Sqrt(rdata.IZZcg/rdata.Area);;
  rdata.rxax:= Sqrt(rdata.IXax/rdata.Area);     //radii of giration from axes
  rdata.ryax:= Sqrt(rdata.IYax/rdata.Area);
  rdata.roax:= Sqrt(rdata.IZZax/rdata.Area);

  if (rdata.IXcg-rdata.IYcg) <> 0 then  begin //principal inertial axes I1 and I2
      rdata.Theta:= ArcTan(2*rdata.IXYcg/(rdata.IXcg-rdata.IYcg))/2; // angle of principal inertia axes
      rdata.I1:= (rdata.IXcg+rdata.IYcg)/2 + Sqrt(Sqr((rdata.IXcg-rdata.IYcg)/2)+sqr(rdata.IXYcg));
      rdata.I2:= (rdata.IXcg+rdata.IYcg)/2 - Sqrt(Sqr((rdata.IXcg-rdata.IYcg)/2)+sqr(rdata.IXYcg));
  end;
   // section modulus local cg
  rdata.WxUpcg:= rdata.IXcg/Abs(rdata.MaxY-rdata.Yax);
  rdata.WxDowncg:= rdata.IXcg/Abs(rdata.Yax-rdata.Miny);
  rdata.WyLeftcg:= rdata.IYcg/Abs(rdata.Xax-rdata.MinX);
  rdata.WyRightcg:= rdata.IYcg/Abs(rdata.MaxX-rdata.Xax);
  rdata.WxMax:= Max(rdata.WxUpcg,rdata.WxDowncg);
  rdata.WyMax:= Max(rdata.WyRightcg,rdata.WyLeftcg);

  CadEntityPutExData(hEnt,@rdata, SizeOf(rdata));
end;

procedure Solverect(hEnt: Thandle);
var
  rdata: TLocalShapeData;
begin
  FillChar(rdata, SizeOf(rdata), 0);  //erase any rdata content
  rdata.iEnt:= CAD_ENT_RECT;
  rdata.ShapeFull:= bFullShape;
  CadCmdPrompt(hDwg, 'solve rect', 'xxxx');
  CadRectGetCenter(hEnt, @rdata.Xax, @rdata.Yax, @rdata.Zax);  //Get rect center coords
  rdata.Width:= CadRectGetWidth(hEnt);  //Get rect width
  rdata.Height:= CadRectGetHeight(hEnt);  //Get rect height
  rdata.Angle:= CadRectGetAngle(hEnt);  //get rect angle
  rdata.Area:= rdata.Width*rdata.Height; //calc rect area
  rdata.Perimeter:= 2*rdata.Height + 2*rdata.Width; //calc rect perimeter
  rdata.IXcg:= (rdata.Width * Power(rdata.Height, 3))/12;
  rdata.IYcg:= (rdata.Height * Power(rdata.Width,3))/12;
  rdata.IXYcg:= 0; //(Power(rdata.Width,2) * Power(rdata.Height, 2))/4;
  //get min and max coords
  rdata.MaxY:= CadEntityGetTop(hEnt);
  rdata.MinY:= CadEntityGetBottom(hEnt);
  rdata.MaxX:= CadEntityGetRight(hEnt);
  rdata.MinX:= CadEntityGetLeft(hEnt);
  //rdata.IXYax:=
  if rdata.Angle <> 0 then begin
     //Rotate inertia
    rdata.IUcg:= (rdata.IXcg + rdata.IYcg)/2 + (rdata.IXcg - rdata.IYcg)/2 * cos(2*rdata.Angle) - rdata.IXYcg * sin(2*rdata.Angle);
    rdata.IVcg:= (rdata.IXcg + rdata.IYcg)/2 - (rdata.IXcg - rdata.IYcg)/2 * cos(2*rdata.Angle) + rdata.IXYcg * sin(2*rdata.Angle);
    rdata.IUVcg:= (rdata.IXcg - rdata.IYcg)/2 * sin(2*rdata.Angle) + rdata.IXYcg * cos(2*rdata.Angle);

    //rdata.rucg:= Sqrt(rdata.IUcg/rdata.Area);
    //rdata.rvcg:= Sqrt(rdata.IVcg/rdata.Area);
    //rdata.IJUVcg:= rdata.IUcg + rdata.IVcg;
    //rdata.rouvcg:= Sqrt(rdata.IJUVcg/rdata.Area);

    rdata.IXcg:= rdata.IUcg;
    rdata.IYcg:= rdata.IVcg;
    rdata.IXYcg:= rdata.IUVcg;
  end;


  rdata.IZZcg:= rdata.IXcg + rdata.IYcg;
  rdata.rxcg:= Sqrt(rdata.IXcg/rdata.Area); //radii of giration shape
  rdata.rycg:= Sqrt(rdata.IYcg/rdata.Area);

  // section modulus local cg
  rdata.WxUpcg:= rdata.IXcg/Abs(rdata.MaxY-rdata.Yax);
  rdata.WxDowncg:= rdata.IXcg/Abs(rdata.Yax-rdata.Miny);
  rdata.WyLeftcg:= rdata.IYcg/Abs(rdata.Xax-rdata.MinX);
  rdata.WyRightcg:= rdata.IYcg/Abs(rdata.MaxX-rdata.Xax);
  rdata.WxMax:= Max(rdata.WxUpcg,rdata.WxDowncg);
  rdata.WyMax:= Max(rdata.WyRightcg,rdata.WyLeftcg);

//  rdata.IUcg:= rdata.IXcg;
//  rdata.IVcg:= rdata.IYcg;
//  rdata.IUVcg:= rdata.IXYcg;


  rdata.IXax:= rdata.IXcg + rdata.Area*Sqr(rdata.Yax);
  rdata.IYax:= rdata.IYcg + rdata.Area*Sqr(rdata.Xax);
  rdata.IZZax:= rdata.IXax + rdata.IYax;
  rdata.IXYax:= rdata.IXYcg + rdata.Area*rdata.Xax*rdata.Yax;;
  rdata.rxax:= Sqrt(rdata.IXax/rdata.Area);     //radii of giration from axes
  rdata.ryax:= Sqrt(rdata.IYax/rdata.Area);



  rdata.MsX:= rdata.Area * rdata.Yax;
  rdata.MsY:= rdata.Area * rdata.Xax;

  CadEntityPutExData(hEnt,@rdata, SizeOf(rdata));
end;

procedure SolveCircle(hEnt: Thandle);
var
  rdata: TLocalShapeData;
begin
 FillChar(rdata, SizeOf(rdata), 0);  //erase any rdata content
 CadCmdPrompt(hDwg, 'solve circle', 'xxxx');
 with rdata do begin
  iEnt:= CAD_ENT_ARCS_CIRCLE;
  ShapeFull:= bFullShape;
  CadArcGetCenter(hEnt, @Xax, @Yax, @Zax);
  Radius:= CadArcGetRadius(hEnt);
  Area:= Pi*Power(Radius,2);
  Perimeter:= 2*pi*Radius;
  IXcg:= pi*Power(Radius,4)/4;
  IYcg:= IXcg;
  IXYcg:=0;
  IZZcg:= IXcg + IYcg;

  MaxY:= CadEntityGetTop(hEnt);
  MinY:= CadEntityGetBottom(hEnt);
  MaxX:= CadEntityGetRight(hEnt);
  MinX:= CadEntityGetLeft(hEnt);

  IXax:= IXcg + Area*Sqr(Yax);
  IYax:= IYcg + Area*Sqr(Xax);
  IXYax:= IXYcg + Area*Xax*Yax;;
  IZZax:=IXax + IYax;
  rxcg:= Sqrt(IXcg/Area); //radii of giration shape
  rycg:= Sqrt(IYcg/Area);
  rxax:= Sqrt(IXax/Area);     //radii of giration from axes
  ryax:= Sqrt(IYax/Area);

    // section modulus local cg
  rdata.WxUpcg:= rdata.IXcg/Abs(rdata.MaxY-rdata.Yax);
  rdata.WxDowncg:= rdata.WxUpcg;
  rdata.WyLeftcg:= rdata.WxUpcg;
  rdata.WyRightcg:= rdata.WxUpcg;
  rdata.WxMax:= rdata.WxUpcg;
  rdata.WyMax:=rdata.WxUpcg;

  MsX:= Area * Yax;
  MsY:= Area * Xax;

  IUcg:=IXcg;
  IVcg:=IYcg;
  IUVcg:=IXYcg;
  IJUVcg:= IUcg + IVcg;

  rucg:= rxcg; //radii of giration shape
  rvcg:= rycg;
  rouvcg:= rxcg;
  Angle:=0; // allways zero in a circle
 end;
 CadEntityPutExData(hEnt,@rdata, SizeOf(rdata));
end;

procedure SolveEllipse(hEnt: Thandle);
var
  rdata: TLocalShapeData;
  h,v: double; // horizontal, vertival ellipse radius
begin
 CadCmdPrompt(hDwg, 'solve ellipse', 'xxxx');
 FillChar(rdata, SizeOf(rdata), 0);  //erase any rdata content
 with rdata do begin
  iEnt:= CAD_ENT_ARCS_ELLIPSE;
  ShapeFull:= bFullShape;
  CadArcGetCenter(hEnt, @Xax, @Yax, @Zax);
  v:= CadArcGetRadHor(hEnt);
  h:= CadArcGetRadVer(hEnt);
  Angle:= CadArcGetRotAngle(hEnt);    //angle in radians
  Perimeter:= 2*pi*Sqrt((Sqr(v)+Sqr(h))/2);
  Area:= pi*v*h;
  IXcg:= 1/4*pi*v*Power(h,3);
  IYcg:= 1/4*pi*h*Power(v,3);
  IXYcg:=0;
  MaxY:= CadEntityGetTop(hEnt);
  MinY:= CadEntityGetBottom(hEnt);
  MaxX:= CadEntityGetRight(hEnt);
  MinX:= CadEntityGetLeft(hEnt);

  if Angle <> 0 then begin
     //Rotate inertia
    IUcg:= (IXcg + IYcg)/2 + (IXcg - IYcg)/2 * cos(2*Angle) - IXYcg * sin(2*Angle);
    IVcg:= (IXcg + IYcg)/2 - (IXcg - IYcg)/2 * cos(2*Angle) + IXYcg * sin(2*Angle);
    IUVcg:= (IXcg - IYcg)/2 * sin(2*Angle) + IXYcg * cos(2*Angle);

  //  rucg:= Sqrt(IUcg/Area);
  //  rvcg:= Sqrt(IVcg/Area);
  //  IJUVcg:= IUcg + IVcg;
  //  rouvcg:= Sqrt(IJUVcg/Area);

    rdata.IXcg:= rdata.IUcg;
    rdata.IYcg:= rdata.IVcg;
    rdata.IXYcg:= rdata.IUVcg;

  end;
  IZZcg:= IXcg + IYcg;
  rxcg:= Sqrt(IXcg/Area); //radii of giration shape
  rycg:= Sqrt(IYcg/Area);

    // section modulus local cg
  rdata.WxUpcg:= rdata.IXcg/Abs(rdata.MaxY-rdata.Yax);
  rdata.WxDowncg:= rdata.IXcg/Abs(rdata.Yax-rdata.Miny);
  rdata.WyLeftcg:= rdata.IYcg/Abs(rdata.Xax-rdata.MinX);
  rdata.WyRightcg:= rdata.IYcg/Abs(rdata.MaxX-rdata.Xax);
  rdata.WxMax:= Max(rdata.WxUpcg,rdata.WxDowncg);
  rdata.WyMax:= Max(rdata.WyRightcg,rdata.WyLeftcg);

  //IUcg:= IXcg;
  //IVcg:= IYcg;
  //IUVcg:= IXYcg;
  //IJUVcg:= IZZcg;
  //rucg:= rxcg; //radii of giration shape
  //rvcg:= rycg;
  //IJUVcg:= IUcg + IVcg;
  //rouvcg:= Sqrt(IJUVcg/Area);;

  IXax:= IXcg + Area*Sqr(Yax);
  IYax:= IYcg + Area*Sqr(Xax);
  IXYax:= IXYcg + Area*Xax*Yax;
  IZZax:= IXax + IYax;
  rxax:= Sqrt(IXax/Area);     //radii of giration from axes
  ryax:= Sqrt(IYax/Area);

  MsX:= Area * Yax;
  MsY:= Area * Xax;
 end;
 CadEntityPutExData(hEnt,@rdata, SizeOf(rdata));
end;

procedure ExecuteShapeProp(nEnts: Integer);
var
  buffLName: TStrBuff;
  sLName: String;
  nBadShapes: UINT;
begin
 nBadShapes:=0;
 if nEnts <= 0 then exit;
     CadCmdPrompt(hDwg, 'start', 'xxxx');
     pEnt:= CadSelGetFirstPtr(hDwg);
     FillChar(ComposedShapeData, SizeOf(ComposedShapeData), 0); // erase all values into ComposedShapeData;
     while pEnt <> 0 do begin
        hEnt:= CadGetEntityByPtr(pEnt);  // get ent handle from pointer
        pEnt:= CadSelGetNextPtr( hDwg, pEnt ); // get next pointer
        idObj:= CadEntityGetLayerID(hEnt); // get entities layer id
        hLayer:= CadGetLayerByID(hDwg, idObj); //get layer handle by layer id
        CadLayerGetName(hLayer, @buffLName);
        slName:= buffLName;
        if (sLName = 'Full_Shape') or (sLName = 'Hollow_Shape') then begin
         bFullShape:= sLName = 'Full_Shape'; //current shapeis full or not
         EntType:= CadEntityGetTypeEx(hEnt);
            case EntType of
                 CAD_ENT_POLYLINE: SolvePolyline(hEnt); //Polyline or Polygon
                 CAD_ENT_RECT: SolveRect(hEnt); // Rectangle
                 CAD_ENT_ARCS_CIRCLE: SolveCircle(hEnt);
                 CAD_ENT_ARCS_ELLIPSE: SolveEllipse(hEnt);
            else
                //case enttype else
            inc(nBadShapes,1);
            end;
        end else
         // inc(nBadShapes,1);    //if slname
     end; // end while

if nBadShapes > 0 then CadCmdPrompt(hDwg, StrToBuffPtr(Format('errors: <%u> unclosed or unsolvable entities selected',[nBadShapes]), buffLName), 'xxxx');

CadCmdPrompt(hDwg, StrToBuffPtr('Double click shapes do view results', buffLName), 'xxxx');
end; // end proc

//Create command
procedure CreateShapePropCommand;
var
  Cursor: HCURSOR;
begin
 Cursor:= 0;
 Cursor:= LoadCursor(HInstance, 'SHAPECUR');
 if not(Boolean(CadAddCommand (hDwg, CMD_DOSHAPEPROP, 'SHAPEPROP', 'SP', Cursor, @DoShapeProp, @DragShapeProp))) then
    ShowMessage(STR_ERROR_FAIL_CUST_SHAPE_COM);

end;

//Do (execute) command
procedure DoShapeProp; stdcall;
var
 nEnts: integer;
 s: TStrBuff;

begin
 nEnts:= CadSelCount(hDwg);

 if not Boolean(nEnts) then begin
    CadCmdPrompt(hDwg, 'Select entities (right click to continue, [ESC] to cancel', 'xxxx');
    nEnts:= CadCmdUserSelect (hDwg);       // start user selection //simple selection mode
 end;


 case nEnts of
 -1: begin
      CadCmdPrompt(hDwg, 'Canceled', 'xxxx');
     end;
  0: begin
      CadCmdPrompt(hDwg, 'no selection', 'xxxx');
     end;
 else
    CadCmdPrompt(hDwg, StrToBuffPtr(Format('Entities selected: (%d)', [nEnts]), s), 'xxxx');
    ExecuteShapeProp(nEnts);
    EntChangeProc(hDWG, 0);
    // do something
 end;

 //if not Boolean(nEnts) then goto lEnd; // start selectio and if user cancel jump to end

 //Clear selection
 CadSelClear(hDwg);
 // update drawing
 CadUpdate(hDwg);
 //CadExecute(hDWG, hWND, CAD_CMD_RESET);
 end;

// drag command (draw while dragging)
procedure DragShapeProp(hDwg: Integer; nPoints: Integer; x, y, z : Double); stdcall;
var
  x0,y0,z0: Double;
begin
  if nPoints = 1 then begin // pelo menos um ponto está no buffer
    CadCmdGetPoint( hDwg, 0, @x0, @y0, @z0 );
     // draw temporary rectangle
    CadDrawAddVertex( x0, y0, z );
    CadDrawAddVertex( x , y0, z );
    CadDrawAddVertex( x , y , z );
    CadDrawAddVertex( x0, y , z );
    CadDrawAddVertex( x0, y0, z );
    CadDrawPolyline();
  end;
end;


end.

