unit uinformation;

{$mode ObjFPC}{$H+}

interface

uses
  Classes, SysUtils, Forms, Controls, Graphics, Dialogs, StdCtrls, ExtCtrls,
  Spin, Buttons, Windows;

type

  { TfrmShowData }

  TfrmShowData = class(TForm)
    Button1: TButton;
    Edit1: TEdit;
    Label1: TLabel;
    Label2: TLabel;
    memoinf: TMemo;
    Panel1: TPanel;
    SpeedButton1: TSpeedButton;
    SpinEdit1: TSpinEdit;
    procedure Button1Click(Sender: TObject);
    procedure Edit1Change(Sender: TObject);
    procedure FormShow(Sender: TObject);
    procedure SpeedButton1Click(Sender: TObject);
    procedure SpinEdit1Change(Sender: TObject);
  private

  public
    procedure ShowData(hEnt: Thandle);
    procedure FormatRectData;
    procedure InsertText(ftext: String);
    function GetShapeTypetag(iType: Integer): String;
  end;

const
  separatorStr = '************    ************';
  FullStr = 'Full shape';
  HollowStr = 'Hollow shape';

var
  frmShowData: TfrmShowData;

implementation
uses uGeoProps, ucustomcomm, CadApi;
{$R *.frm}

{ TfrmShowData }
 var
  rShapeExData: TLocalShapeData; //record with shape datum

procedure TfrmShowData.Button1Click(Sender: TObject);
begin
  Close;
end;

procedure TfrmShowData.Edit1Change(Sender: TObject);
begin
  custUnit:= Edit1.Text;
end;

procedure TfrmShowData.FormShow(Sender: TObject);
begin
  SpinEdit1.Value:= custPrecision;
  Edit1.Text:= custUnit;
end;

procedure TfrmShowData.SpeedButton1Click(Sender: TObject);
begin
  ShowData(hEnt );
end;

procedure TfrmShowData.SpinEdit1Change(Sender: TObject);
begin
  custPrecision:= SpinEdit1.Value;
end;

procedure TfrmShowData.ShowData(hEnt: Thandle);
var
 i: Integer;
begin
  FillChar(rShapeExData, SizeOf(rShapeExData), 0); //clear all data into var
  i:= CadEntityGetExDataSize(hEnt);
  if i= 0 then exit;
  memoinf.Clear; // clear memo
  CadEntityGetExData(hEnt, @rShapeExData);  //get extended data from entity
  FormatRectData;
  Show;
end;

procedure TfrmShowData.FormatRectData;
var
 s, ps: String;  //string and precigion str
begin
s:= '';
ps:= ' %8.' + IntToStr(custPrecision) + 'f';
 with rShapeExData do begin
  InsertText('***** Shapes basic data local coordinates *****');
  InsertText(GetShapeTypetag(iEnt));
  s:= 'Shape nature solid/hollow: ';
  if ShapeFull then s:= s + FullStr
               else s:= s + HollowStr;
  InsertText(s);
  s:= Format('Area:' + ps,[Area]);
  InsertText(s);
  s:= Format('Perimeter:' + ps,[Perimeter]);
  InsertText(s);
  s:= Format('Ix (local cg):' + ps,[IXcg]);
  InsertText(s);
  s:= Format('Iy (local cg):' + ps,[IYcg]);
  InsertText(s);
  s:= Format('Ixy (local cg):' + ps,[IXYcg]);
  InsertText(s);
  s:= Format('Izz/J (local cg):' + ps,[IZZcg]);
  InsertText(s);
  s:= Format('Rx (local):' + ps,[rxcg]);
  InsertText(s);
  s:= Format('Ry (local):' + ps,[rycg]);
  InsertText(s);
  s:= 'Section Modulus:';
  InsertText(s);
  s:= Format('Wx_Up:' + ps +', Wx_Bottom:' + ps + ', Wy_Left:' + ps + ', Wy_Right:' + ps,[WxUpcg,WxDowncg,WyLeftcg,WyRightcg]);
  InsertText(s);
  s:= Format('Wx_Max:' + ps + ', Wy_Max:' + ps,[WxMax,WyMax]);
  InsertText(s);
  InsertText(separatorStr);
  InsertText(' ');
  if iEnt = CAD_ENT_POLYLINE then begin
    InsertText('***** Principal axis of innertia I1, I2 *****');
    s:= Format('I1:' + ps,[I1]);
    InsertText(s);
    s:= Format('I2:' + ps,[I2]);
    InsertText(s);
    s:= Format('Theta:' + ps,[Theta * 57.295779513]);
    InsertText(s);
    InsertText(' ');
  end else if Angle > 0 then begin
    InsertText('***** Rotated axis of inertia IU, IV  *****');
    s:= Format('Rotation angle:' + ps + ' (degrees)',[Angle* 57.295779513]);
    InsertText(s);
    s:= Format('Iu (I1, local cg):' + ps,[IUcg]);
    InsertText(s);
    s:= Format('Iv (I2, local cg):' + ps,[IVcg]);
    InsertText(s);
    s:= Format('Iuv (local cg):' + ps,[IUVcg]);
    InsertText(s);
    s:= Format('Juv (local cg):' + ps,[IJUVcg]); // polar inertia
    InsertText(s);
    s:= Format('ru (local cg):' + ps,[rucg]);
    InsertText(s);
    s:= Format('rv (local cg):' + ps,[rvcg]);
    InsertText(s);
    InsertText(separatorStr);
    InsertText(' ');
  end;
  InsertText('***** Shape data relative to (0,0) global coordinates *****');
  s:= Format('Centroid: (' + ps + ',' + ps + ')',[Xax, Yax]);
  InsertText(s);
  s:= Format('Ix (global):' + ps,[IXax]);
  InsertText(s);
  s:= Format('Iy (global):' + ps,[IYax]);
  InsertText(s);
  s:= Format('Ixy (global):' + ps,[IXYax]);
  InsertText(s);
  s:= Format('Izz/J (global):' + ps,[IZZax]);
  InsertText(s);
  s:= Format('Rx (global):' + ps,[rxax]);
  InsertText(s);
  s:= Format('Ry (global):' + ps,[ryax]);
  InsertText(s);
  s:= Format('Static moments MsX: ' + ps + ' , MsY: ' + ps,[Msx, MsY]);
  InsertText(s);
  InsertText(separatorStr);
  end; // end with


end;

procedure TfrmShowData.InsertText(ftext: String);
begin
  memoinf.Lines.Add(ftext);
end;

function TfrmShowData.GetShapeTypetag(iType: Integer): String;
begin
  result:= 'Shape type: ';
  case iType of
    CAD_ENT_POLYLINE: result:= result + 'polyline';
    CAD_ENT_RECT: result:= result + 'rectangle';
    CAD_ENT_ARCS_CIRCLE: result := result + 'circle';
    CAD_ENT_ARCS_ELLIPSE: result := result + 'ellipse';
    else
      result:= '';
  end;
end;

end.

